/**************************************************************************//**
*   @file   AD5111.c
*   @brief  Implementation of AD5111 Driver for Nios II processor.
*   @author ATofan (alexandru.tofan@analog.com)
*
*******************************************************************************
* Copyright 2011(c) Analog Devices, Inc.
*
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*  - Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
*  - Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in
*    the documentation and/or other materials provided with the
*    distribution.
*  - Neither the name of Analog Devices, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived
*    from this software without specific prior written permission.
*  - The use of this software may or may not infringe the patent rights
*    of one or more patent holders.  This license does not release you
*    from the requirement that you obtain separate licenses from these
*    patent holders to use this software.
*  - Use of the software either in source or binary form, must be run
*    on or directly connected to an Analog Devices Inc. component.
*
* THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR IMPLIED
* WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT, MERCHANTABILITY
* AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*******************************************************************************
*   SVN Revision: 468
******************************************************************************/
#define AD5111_GLOBALS
#define GPIO_BASEADDR GPIO_BASE
#define CS 1
#define UnD 2
#define CLK 4

/*****************************************************************************/
/* Include Files                                                             */
/*****************************************************************************/
#include "../ADI_Driver/AD5111.h"
#include "system.h"
#include "io.h"
#include "altera_avalon_pio_regs.h"

/*****************************************************************************/
/* Aliases for uC/Probe data                                                 */
/*****************************************************************************/
volatile char* AD5111UpDownBtn		= &AD5111_Data[0].chData;
volatile char* AD5111EnBtn   		= &AD5111_Data[1].chData;
volatile char* AD5111RValue			= &AD5111_Data[2].chData;
volatile char* AD5111ModeBtn		= &AD5111_Data[3].chData;
volatile unsigned char* AD5111Pos	= &AD5111_Data[4].uchData;
char AD5111PrevEnBtn = 0;

void incValue(void)
{
	//GPIO_CLEAR_BITS(GPIO_BASEADDR, CLK);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASEADDR, CLK);
	//GPIO_SET_BITS(GPIO_BASEADDR, UnD);
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASEADDR, UnD);
	//GPIO_CLEAR_BITS(GPIO_BASEADDR, CS);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASEADDR, CS);

	//GPIO_SET_BITS(GPIO_BASEADDR, CLK);
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASEADDR, CLK);
	//GPIO_CLEAR_BITS(GPIO_BASEADDR, CLK);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASEADDR, CLK);
	//GPIO_SET_BITS(GPIO_BASEADDR, CLK);
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASEADDR, CLK);

	//GPIO_SET_BITS(GPIO_BASEADDR, CS);
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASEADDR, CS);
}

void decValue(void)
{
	//GPIO_CLEAR_BITS(GPIO_BASEADDR, CLK);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASEADDR, CLK);
	//GPIO_CLEAR_BITS(GPIO_BASEADDR, UnD);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASEADDR, UnD);
	//GPIO_CLEAR_BITS(GPIO_BASEADDR, CS);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASEADDR, CS);

	//GPIO_SET_BITS(GPIO_BASEADDR, CLK);
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASEADDR, CLK);
	//GPIO_CLEAR_BITS(GPIO_BASEADDR, CLK);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASEADDR, CLK);
	//GPIO_SET_BITS(GPIO_BASEADDR, CLK);
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASEADDR, CLK);

	//GPIO_SET_BITS(GPIO_BASEADDR, CS);
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASEADDR, CS);
}

void shutDown(void)
{
	//GPIO_CLEAR_BITS(GPIO_BASEADDR, UnD);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASEADDR, UnD);
	//GPIO_SET_BITS(GPIO_BASEADDR, CLK);
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASEADDR, CLK);
	//GPIO_CLEAR_BITS(GPIO_BASEADDR, CS);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASEADDR, CS);
	//GPIO_SET_BITS(GPIO_BASEADDR, UnD);
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASEADDR, UnD);
}

void saveMem(void)
{
	//GPIO_CLEAR_BITS(GPIO_BASEADDR, UnD);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASEADDR, UnD);
	//GPIO_SET_BITS(GPIO_BASEADDR, CLK);
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASEADDR, CS);
	//GPIO_CLEAR_BITS(GPIO_BASEADDR, CS);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASEADDR, CS);
	//GPIO_SET_BITS(GPIO_BASEADDR, UnD);
	IOWR_ALTERA_AVALON_PIO_SET_BITS(GPIO_BASEADDR, UnD);
	//GPIO_CLEAR_BITS(GPIO_BASEADDR, UnD);
	IOWR_ALTERA_AVALON_PIO_CLEAR_BITS(GPIO_BASEADDR, UnD);
}

int AD5111_Init(void)
{
	*AD5111Pos = 0;
    // Set GPIO0 GPIO1 and GPIO2 as outputs
	IOWR_ALTERA_AVALON_PIO_DIRECTION(GPIO_BASEADDR, 0x07);
    //GPIO_SET_OUTPUT_PINS(GPIO_BASEADDR, 0xF8);
	return(0);
}

int AD5111_UpdateParameters(void)
{
	int i = 0;

	if(*AD5111ModeBtn == 1)
	{
		if(*AD5111Pos < *AD5111RValue)
		{
			for(i = *AD5111Pos; i < *AD5111RValue; i++)
			{
				incValue();
				*AD5111Pos = *AD5111Pos + 1;
			}
		}
		else if(*AD5111Pos > *AD5111RValue)
		{
			for(i = *AD5111Pos; i > *AD5111RValue; i--)
			{
				decValue();
				*AD5111Pos = *AD5111Pos - 1;
			}
		}
	}
	else if(*AD5111ModeBtn == 0)
	{
		if(*AD5111EnBtn != AD5111PrevEnBtn)
		{
			if((*AD5111UpDownBtn == 1)&&(*AD5111Pos < 128))
			{
				incValue();
				*AD5111Pos = *AD5111Pos + 1;
			}
			else if((*AD5111UpDownBtn == 0)&&(*AD5111Pos > 0))
			{
				decValue();
				*AD5111Pos = *AD5111Pos - 1;
			}
			AD5111PrevEnBtn = *AD5111EnBtn;
		}
	}
	return(0);
}
